@@ -2,7 +2,7 @@
 
 ;;; File: "_gambit#.scm"
 
-;;; Copyright (c) 1994-2019 by Marc Feeley, All Rights Reserved.
+;;; Copyright (c) 1994-2022 by Marc Feeley, All Rights Reserved.
 
 ;;;============================================================================
 
@@ -75,7 +75,7 @@
 
 ;; Special objects.
 
-(##define-macro (macro-absent-obj)   `',c#absent-object) ;;(##type-cast -6 (##type #f)))
+(##define-macro (macro-absent-obj)   `',(##absent-object))
 (##define-macro (macro-unbound1-obj) `',(##type-cast -7 (##type #f)))
 (##define-macro (macro-unbound2-obj) `',(##type-cast -8 (##type #f)))
 (##define-macro (macro-unused-obj)   `',(##type-cast -14 (##type #f)))
@@ -141,6 +141,496 @@
                (extended-bindings))
              val))))))
 
+(##define-syntax declare-safe-define-procedure
+  (lambda (src)
+
+    (define (err)
+      (##raise-expression-parsing-exception
+       'ill-formed-special-form
+       src
+       (##source-strip (car (##source-strip src)))))
+
+    (##deconstruct-call
+     src
+     2
+     (lambda (arg-src)
+       (let ((arg (##desourcify arg-src))
+             (meta-info-tbl (vector-ref (##compilation-ctx) 2)))
+         (if arg
+             (table-set! meta-info-tbl 'safe-define-procedure arg)
+             (if (eq? (table-ref meta-info-tbl 'safe-define-procedure #f)
+                      #t)
+                 (table-set! meta-info-tbl 'safe-define-procedure))))
+       `(##begin)))))
+
+(##define-syntax standard
+  (lambda (src)
+
+    (define (err)
+      (##raise-expression-parsing-exception
+       'ill-formed-special-form
+       src
+       (##source-strip (car (##source-strip src)))))
+
+    (define (proc sym)
+      `(##let () (##namespace ("" ,sym)) ,sym))
+
+    (##deconstruct-call
+     src
+     2
+     (lambda (arg-src)
+       (let ((arg (##desourcify arg-src)))
+         (cond ((symbol? arg)
+                (proc arg))
+               ((pair? arg)
+                (let ((name (##desourcify (car arg))))
+                  (if (symbol? name)
+                      (cons (proc name) (cdr arg))
+                      (err))))
+               (else
+                (err))))))))
+
+(##define-syntax primitive
+  (lambda (src)
+
+    (define (err)
+      (##raise-expression-parsing-exception
+       'ill-formed-special-form
+       src
+       (##source-strip (car (##source-strip src)))))
+
+    (define (symbol-append . lst)
+      (string->symbol (string-concatenate (map symbol->string lst))))
+
+    (define (prim sym)
+      (symbol-append '|##| sym))
+
+    (##deconstruct-call
+     src
+     2
+     (lambda (arg-src)
+       (let ((arg (##desourcify arg-src)))
+         (cond ((symbol? arg)
+                (prim arg))
+               ((pair? arg)
+                (let ((name (##desourcify (car arg))))
+                  (if (symbol? name)
+                      (cons (prim name) (cdr arg))
+                      (err))))
+               (else
+                (err))))))))
+
+(##define-syntax define-procedure
+  (lambda (src)
+
+    (define (err)
+      (##raise-expression-parsing-exception
+       'ill-formed-special-form
+       src
+       (##source-strip (car (##source-strip src)))))
+
+    (define (safe-define-procedure?)
+      (let* ((cc
+              (##global-var-ref
+               (##make-global-var '##compilation-ctx)))
+             (meta-info-tbl
+              (and (##not (##unbound? cc))
+                   (vector-ref (cc) 2))))
+        (and meta-info-tbl
+             (table-ref meta-info-tbl 'safe-define-procedure #f))))
+
+    (define (expand src)
+      (##deconstruct-call
+       src
+       -2
+       (lambda (pattern-src . body)
+         (let* ((pattern (##source-strip pattern-src))
+                (head (and (pair? pattern) (##source-strip (car pattern))))
+                (prim? (and (pair? head)
+                            (pair? (cdr head))
+                            (memq (##source-strip (car head))
+                                  '(primitive prim&proc))))
+                (name (if prim?
+                          (##source-strip (cadr head))
+                          head)))
+           (if (not (symbol? name))
+               (##raise-expression-parsing-exception
+                'ill-formed-special-form
+                src
+                (##source-strip (car (##source-strip src))))
+               (parse-parameters (and prim? (car prim?))
+                                 name
+                                 body
+                                 (cdr pattern)))))))
+
+    (define object-check '(object))
+    (define strict-object-check '(strict-object))
+    (define aux-suffix '-aux)
+
+    (define (parse-check check-src)
+      (let ((check (##source-strip check-src)))
+        (cond ((symbol? check)
+               (list check))
+              ((and (pair? check)
+                    (symbol? (##source-strip (car check))))
+               (cons (##source-strip (car check))
+                     (cdr check)))
+              (else
+               #f))))
+
+    (define (parse-type type-src)
+      (let ((type (##source-strip type-src)))
+        (if (and (pair? type)
+                 (eq? 'and (##source-strip (car type))))
+            (let loop ((lst (cdr type)) (rev-checks '()))
+              (if (pair? lst)
+                  (let ((check (parse-check (car lst))))
+                    (and check
+                         (loop (cdr lst) (cons check rev-checks))))
+                  (reverse rev-checks)))
+            (let ((check (parse-check type-src)))
+              (and check
+                   (list check))))))
+
+    (define (parse-parameters prim? name body params)
+      (let loop ((lst params)
+                 (rev-req '())
+                 (rev-opt '()))
+
+        (define (parse-rest-param param)
+          (cond ((symbol? param)
+                 (process-parameters prim?
+                                     name
+                                     body
+                                     (reverse rev-req)
+                                     (reverse rev-opt)
+                                     (vector param (list object-check))))
+                ((pair? param)
+                 (let ((len (##proper-length param)))
+                   (if (eqv? len 2)
+                       (let* ((var-src (car param))
+                              (var (##source-strip var-src))
+                              (checks (parse-type (cadr param))))
+                         (if checks
+                             (process-parameters prim?
+                                                 name
+                                                 body
+                                                 (reverse rev-req)
+                                                 (reverse rev-opt)
+                                                 (vector var checks))
+                             (err)))
+                       (err))))
+                (else
+                 (err))))
+
+        (cond ((null? lst)
+               (process-parameters prim?
+                                   name
+                                   body
+                                   (reverse rev-req)
+                                   (reverse rev-opt)
+                                   #f))
+              ((pair? lst)
+               (let* ((param-src (car lst))
+                      (param (##source-strip param-src)))
+                 (cond ((and (pair? (cdr lst))
+                             (null? (cddr lst))
+                             (eq? (##source-strip (cadr lst)) '...))
+                        (parse-rest-param param))
+                       ((and (null? rev-opt) (symbol? param))
+                        (loop (cdr lst)
+                              (cons (vector param (list object-check))
+                                    rev-req)
+                              rev-opt))
+                       ((pair? param)
+                        (let ((len (##proper-length param)))
+                          (if (or (and (eqv? len 2) (null? rev-opt))
+                                  (eqv? len 3))
+                              (let* ((var-src (car param))
+                                     (var (##source-strip var-src))
+                                     (checks (parse-type (cadr param))))
+                                (cond ((not checks)
+                                       (err))
+                                      ((eqv? len 3)
+                                       (let ((default-src (caddr param)))
+                                         (loop (cdr lst)
+                                               rev-req
+                                               (cons (vector var
+                                                             checks
+                                                             default-src)
+                                                     rev-opt))))
+                                      (else
+                                       (loop (cdr lst)
+                                             (cons (vector var
+                                                           checks)
+                                                     rev-req)
+                                             rev-opt))))
+                              (err))))
+                       (else
+                        (err)))))
+              (else
+               (parse-rest-param (##source-strip lst))))))
+
+    (define (var-name param)
+      (if (symbol? param)
+          param
+          (vector-ref param 0)))
+
+    (define (var-checks param)
+      (vector-ref param 1))
+
+    (define (default param)
+      (vector-ref param 2))
+
+    (define (param-name param)
+      (symbol-append '% (var-name param)))
+
+    (define (exception-param-name param)
+      (if (optional? param)
+          (param-name param)
+          (var-name param)))
+
+    (define (optional? param)
+      (and (vector? param)
+           (fx= 3 (vector-length param))))
+
+    (define (force? param)
+      (not (equal? (car (var-checks param)) object-check)))
+
+    (define (symbol-append . lst)
+      (string->symbol (string-concatenate (map symbol->string lst))))
+
+    (define (prim sym)
+      (symbol-append '|##| sym))
+
+    (define (process-parameters prim?
+                                name
+                                body
+                                req-params
+                                opt-params
+                                rest-param)
+
+      (define with-aux?
+        (and (eq? prim? 'prim&proc) rest-param))
+
+      (define (gen-definition kind)
+        (let* ((req-opt-params
+                (append req-params opt-params))
+               (prim-name
+                (prim name))
+               (proc-name
+                (case kind
+                  ((procedure) name)
+                  ((primitive) prim-name)
+                  ((primitive-aux) (prim (symbol-append name aux-suffix))))))
+
+          (define (gen-bind-non-opt expr)
+            (let ((params
+                   (if (not rest-param)
+                       req-params
+                       (append req-params (list rest-param)))))
+              (if (null? params)
+                  expr
+                  `(##let ,(map (lambda (param)
+                                  (list (var-name param) (param-name param)))
+                                params)
+                          ,expr))))
+
+          (define (gen-force expr)
+            (if (eq? kind 'primitive-aux)
+                expr
+                (let ((force-params
+                       (##append-lists
+                        (map (lambda (param)
+                               (if (force? param) (list (param-name param)) '()))
+                             req-opt-params))))
+                  (if (null? force-params)
+                      expr
+                      `(macro-force-vars ,force-params ,expr)))))
+
+          (define (gen-default param expr)
+            (if (optional? param)
+                (let ((def-expr (default param)))
+                  `(##let ((,(var-name param)
+                            ,(if (equal? (##desourcify def-expr)
+                                         '(macro-absent-obj))
+                                 (param-name param)
+                                 `(##if (##eq? ,(param-name param)
+                                               (macro-absent-obj))
+                                        ,def-expr
+                                        ,(param-name param)))))
+                          ,expr))
+                expr))
+
+          (define (gen-check param arg-num check expr)
+            (if (or (equal? check object-check)
+                    (equal? check strict-object-check)
+                    (not (eq? kind 'procedure)))
+                expr
+                `(,(symbol-append 'macro-check- (car check))
+                  ,(var-name param)
+                  '(,arg-num . ,(var-name param))
+                  ,@(cdr check)
+                  ((%procedure%)
+                   ,@(append (map exception-param-name req-opt-params)
+                             (if (not rest-param)
+                                 '()
+                                 (exception-param-name rest-param))))
+                  ,expr)))
+
+          (define (gen-checks param arg-num checks expr)
+            (if (pair? checks)
+                (gen-check param
+                           arg-num
+                           (car checks)
+                           (gen-checks param
+                                       arg-num
+                                       (cdr checks)
+                                       expr))
+                expr))
+
+          (define (gen-defaults-and-checks params arg-num expr)
+            (if (null? params)
+                expr
+                (let ((param (car params)))
+                  (gen-default param
+                               (gen-checks param
+                                           arg-num
+                                           (var-checks param)
+                                           (gen-defaults-and-checks
+                                            (cdr params)
+                                            (fx+ arg-num 1)
+                                            expr))))))
+
+          (define (gen-body)
+            (let ((req (map var-name req-params))
+                  (opt (map var-name opt-params)))
+              (if with-aux?
+                  (if (eq? kind 'primitive-aux)
+                      `(##let ()
+                         ,@body)
+                      `(##let ()
+                         (##declare (not interrupts-enabled))
+                         (,(prim (symbol-append name aux-suffix))
+                          ,@req
+                          ,@opt
+                          ,(var-name rest-param))))
+                  (if (or (null? body)
+                          (and (eq? prim? 'prim&proc)
+                               (eq? kind 'procedure)))
+                      (if (eq? kind 'procedure)
+                          `(##let ()
+                             (##declare (not interrupts-enabled))
+                             (primitive (,proc-name ,@req ,@opt)))
+                          `(##let ()
+                             (##declare (not interrupts-enabled))
+                             (##declare (not safe))
+                             (,proc-name ,@req ,@opt)))
+                      `(##let ()
+                         ,@body)))))
+
+          (define (gen-block expr)
+            expr #; ;; remove after next release
+            `(cond-expand ((compilation-target (_))
+                           ;; avoid ##declare-scope which is not
+                           ;; implemented by interpreter
+                           ,expr)
+                          (else
+                           (##declare-scope
+                            (##begin
+                             (##declare (block))
+                             ,expr)))))
+
+          (let* ((param-list-tail
+                  (if (not rest-param)
+                      '()
+                      (if (eq? kind 'primitive-aux)
+                          (list (param-name rest-param))
+                          (param-name rest-param))))
+                 (param-list
+                  (append
+                   (map param-name req-params)
+                   (if (null? opt-params)
+                       param-list-tail
+                       (cons '#!optional
+                             (append
+                              (map (lambda (param)
+                                     `(,(param-name param)
+                                       (macro-absent-obj)))
+                                   opt-params)
+                              param-list-tail)))))
+                 (expansion
+                  (gen-block
+                   `(##define ,proc-name
+                      (##lambda ,param-list
+                        (##let ((%procedure% (##lambda () ,name)))
+                          (##declare (extended-bindings))
+                          ,@(if (safe-define-procedure?)
+                                `((##declare (safe)))
+                                `((##declare (not safe))
+                                  (##include "~~lib/gambit/prim/prim#.scm")))
+                          ,(gen-force
+                            (gen-bind-non-opt
+                             (gen-defaults-and-checks
+                              req-opt-params
+                              1
+                              (gen-body))))))))))
+            ;;(pp (##desourcify expansion))
+            expansion)))
+
+      (if (and (null? body) rest-param)
+          (error "define-procedure with empty body does not support rest parameter")
+          (if (eq? prim? 'prim&proc)
+              `(##begin
+                ,@(if rest-param
+                      `(,(gen-definition 'primitive-aux))
+                      `())
+                ,(gen-definition 'primitive)
+                ,(gen-definition 'procedure))
+              (gen-definition (or prim? 'procedure)))))
+
+    (expand src)))
+
+(##define-syntax define-primitive
+  (lambda (src)
+
+    (define (err)
+      (##raise-expression-parsing-exception
+       'ill-formed-special-form
+       src
+       (##source-strip (car (##source-strip src)))))
+
+    (##deconstruct-call
+     src
+     -2
+     (lambda (pattern-src . body)
+       (let ((pattern (##desourcify pattern-src)))
+         (cond ((pair? pattern)
+                `(define-procedure ((primitive ,(car pattern)) ,@(cdr pattern))
+                   ,@body))
+               (else
+                (err))))))))
+
+(##define-syntax define-prim&proc
+  (lambda (src)
+
+    (define (err)
+      (##raise-expression-parsing-exception
+       'ill-formed-special-form
+       src
+       (##source-strip (car (##source-strip src)))))
+
+    (##deconstruct-call
+     src
+     -2
+     (lambda (pattern-src . body)
+       (let ((pattern (##desourcify pattern-src)))
+         (cond ((pair? pattern)
+                `(define-procedure ((prim&proc ,(car pattern)) ,@(cdr pattern))
+                   ,@body))
+               (else
+                (err))))))))
+
 ;;;----------------------------------------------------------------------------
 
 ;; Macro to define structure accessors.
@@ -411,7 +901,7 @@
 (macro-define-syntax macro-no-check
   (lambda (stx)
     (syntax-case stx ()
-      ((_ var arg-num form expr)
+      ((_ var arg-id form expr)
        #'expr))))
 
 (##define-macro
@@ -611,139 +1101,118 @@
   (define-prim-nary-bool form zero one two forcing pre-check . post-checks)
   `(define-prim-fold #t ,form ,zero ,one ,two ,forcing ,pre-check ,@post-checks))
 
-(##define-macro (define-check-type type-id type predicate . arguments)
+(##define-macro (define-check-type type-and-test-name type predicate . arguments)
 
-  (define (sym . lst)
-    (string->symbol (apply string-append (map symbol->string lst))))
+  (define (symbol-append . lst)
+    (string->symbol (string-concatenate (map symbol->string lst))))
+
+  (let ((type-name
+         (if (symbol? type-and-test-name)
+             type-and-test-name
+             (car type-and-test-name)))
+        (test-name
+         (if (symbol? type-and-test-name)
+             type-and-test-name
+             (cadr type-and-test-name))))
+
+    (define macro-check-test (symbol-append 'macro-check- test-name))
+    (define macro-test-test (symbol-append 'macro-test- test-name))
+
+    (define macro-fail-check-type (symbol-append 'macro-fail-check- type-name))
+    (define ##fail-check-type (symbol-append '##fail-check- type-name))
+
+    `(begin
+
+       ,@(if type
+             `((##define-macro (,(symbol-append 'implement-check-type- type-name))
+                 '(define-fail-check-type ,type-name ,type))
+               (##define-macro (,macro-fail-check-type arg-id form)
+                 `(macro-handle-failure ,',##fail-check-type ,arg-id ,form)))
+             `())
+
+       (##define-macro (,macro-test-test var)
+         `(,',predicate ,var ,@',arguments))
+
+       (##define-macro (,macro-check-test var arg-id form expr)
+         `(macro-if-checks
+           (if (,',predicate ,var ,@',arguments)
+               ,expr
+               (,',macro-fail-check-type ,arg-id ,form))
+           ,expr)))))
+
+(##define-macro (define-fail-check-type type-name . type-id)
+
+  (define (symbol-append . lst)
+    (string->symbol (string-concatenate (map symbol->string lst))))
 
   (let ()
 
-    (define macro-check-type (sym 'macro-check- type-id))
-    (define macro-test-type (sym 'macro-test- type-id))
-    (define ##fail-check-type (sym '##fail-check- type-id))
+    (define ##fail-check-type (symbol-append '##fail-check- type-name))
 
-    `(begin
-       (##define-macro (,(sym 'implement-check-type- type-id));;;;;;;;;;
-         '(define-fail-check-type ,type-id ,type))
+    `(define-prim (,##fail-check-type arg-id proc . args)
+       (##raise-type-exception
+        arg-id
+        ,(if (pair? type-id) (car type-id) `',type-name)
+        proc
+        args))))
 
-     (##define-macro (,macro-test-type var)
-       `(,',predicate ,var ,@',arguments))
+(##define-macro (macro-handle-failure failure-handler arg-id form)
 
-     (##define-macro (,macro-check-type var arg-num form expr)
+  (define (rest-param x)
+    (if (pair? x)
+        (rest-param (cdr x))
+        x))
 
-       (define (rest-param x)
-         (if (pair? x)
-             (rest-param (cdr x))
-             x))
+  (define (nonrest-params x)
+    (if (pair? x)
+        (cons (car x) (nonrest-params (cdr x)))
+        '()))
 
-       (define (nonrest-params x)
-         (if (pair? x)
-           (cons (car x) (nonrest-params (cdr x)))
-           '()))
+  (define (key-params x)
+    (if (pair? x)
+        (if (keyword? (car x))
+            (cons (car x) (cons (cadr x) (key-params (cddr x))))
+            (key-params (cdr x)))
+        '()))
 
-       (define (key-params x)
-         (if (pair? x)
-           (if (keyword? (car x))
-             (cons (car x) (cons (cadr x) (key-params (cddr x))))
-             (key-params (cdr x)))
-           '()))
+  (define (prekey-params x)
+    (if (or (not (pair? x)) (keyword? (car x)))
+        '()
+        (cons (car x) (prekey-params (cdr x)))))
 
-       (define (prekey-params x)
-         (if (or (not (pair? x)) (keyword? (car x)))
-           '()
-           (cons (car x) (prekey-params (cdr x)))))
-
-       (define (failure name)
-         (let* ((k (key-params (cdr form)))
-                (r (rest-param (cdr form)))
-                (nr (nonrest-params (cdr form)))
-                (pk (prekey-params nr)))
-           (if (and (null? k) (not (null? r)))
-             `(,name ,arg-num '() ,(car form) ,@pk ,r)
-             `(,name
-               ,arg-num
-               ,(if (and (null? k) (null? r))
-                  (car form)
-                  `(##list ,(car form) ,@k ,@(if (null? r) '() (list r))))
-               ,@pk))))
-
-       `(macro-if-checks
-          (if (,',predicate ,var ,@',arguments)
-            ,expr
-            ,(failure ',##fail-check-type))
-          ,expr)))))
+  (let* ((k (key-params (cdr form)))
+         (r (rest-param (cdr form)))
+         (nr (nonrest-params (cdr form)))
+         (pk (prekey-params nr)))
+    (if (and (null? k) (not (null? r)))
+        `(,failure-handler ,arg-id '() ,(car form) ,@pk ,r)
+        `(,failure-handler
+          ,arg-id
+          ,(if (and (null? k) (null? r))
+               (car form)
+               `(##list ,(car form) ,@k ,@(if (null? r) '() (list r))))
+          ,@pk))))
 
 (##define-macro (define-check-index-range-macro type-id predicate . arguments)
 
-  (define (sym . lst)
-    (string->symbol (apply string-append (map symbol->string lst))))
+  (define (symbol-append . lst)
+    (string->symbol (string-concatenate (map symbol->string lst))))
 
   (let ()
 
-    (define macro-check-type (sym 'macro-check- type-id))
-
-    `(##define-macro (,macro-check-type var arg-num ,@arguments form expr)
-
-       (define (rest-param x)
-         (if (pair? x)
-             (rest-param (cdr x))
-             x))
-
-       (define (nonrest-params x)
-         (if (pair? x)
-           (cons (car x) (nonrest-params (cdr x)))
-           '()))
-
-       (define (key-params x)
-         (if (pair? x)
-           (if (keyword? (car x))
-             (cons (car x) (cons (cadr x) (key-params (cddr x))))
-             (key-params (cdr x)))
-           '()))
-
-       (define (prekey-params x)
-         (if (or (not (pair? x)) (keyword? (car x)))
-           '()
-           (cons (car x) (prekey-params (cdr x)))))
-
-       (define (failure name)
-         (let* ((k (key-params (cdr form)))
-                (r (rest-param (cdr form)))
-                (nr (nonrest-params (cdr form)))
-                (pk (prekey-params nr)))
-           `(,name
-             ,arg-num
-             ,(if (and (null? k) (null? r))
-                (car form)
-                `(##list ,(car form) ,@k ,@(if (null? r) '() (list r))))
-             ,@pk)))
+    (define macro-check-type (symbol-append 'macro-check- type-id))
 
+    `(##define-macro (,macro-check-type var arg-id ,@arguments form expr)
        `(macro-if-checks
           (if (##fixnum? ,var)
             (if (,',predicate ,var ,@,(cons 'list arguments))
               ,expr
-              ,(failure '##raise-range-exception))
+              (macro-handle-failure ##raise-range-exception ,arg-id ,form))
             (if (##bignum? ,var)
-              ,(failure '##raise-range-exception)
-              ,(failure '##fail-check-exact-integer)))
+              (macro-handle-failure ##raise-range-exception ,arg-id ,form)
+              (macro-handle-failure ##fail-check-exact-integer ,arg-id ,form)))
           ,expr))))
 
-(##define-macro (define-fail-check-type type-name . type-id)
-
-  (define (sym . lst)
-    (string->symbol (apply string-append (map symbol->string lst))))
-
-  (let ()
-
-    (define ##fail-check-type (sym '##fail-check- type-name))
-
-    `(define-prim (,##fail-check-type arg-num proc . args)
-       (##raise-type-exception
-        arg-num
-        ,(if (pair? type-id) (car type-id) `',type-name)
-        proc
-        args))))
-
 (##define-macro (define-library-type . args)
   `(define-library-type-expand define-type define-library-type ,@args))
 
@@ -978,12 +1447,114 @@
 
 ;;;----------------------------------------------------------------------------
 
+;; u8vector and f64vector are always enabled
+
+(macro-define-syntax macro-if-s8vector
+  (lambda (stx)
+    (syntax-case stx ()
+      ((_ yes)
+       #'(macro-if-s8vector yes (##begin)))
+      ((_ yes no)
+       #'(cond-expand
+          ((or enable-s8vector (not disable-s8vector))
+           yes)
+          (else
+           no))))))
+
+(macro-define-syntax macro-if-u16vector
+  (lambda (stx)
+    (syntax-case stx ()
+      ((_ yes)
+       #'(macro-if-u16vector yes (##begin)))
+      ((_ yes no)
+       #'(cond-expand
+          ((or enable-u16vector (not disable-u16vector))
+           yes)
+          (else
+           no))))))
+
+(macro-define-syntax macro-if-s16vector
+  (lambda (stx)
+    (syntax-case stx ()
+      ((_ yes)
+       #'(macro-if-s16vector yes (##begin)))
+      ((_ yes no)
+       #'(cond-expand
+          ((or enable-s16vector (not disable-s16vector))
+           yes)
+          (else
+           no))))))
+
+(macro-define-syntax macro-if-u32vector
+  (lambda (stx)
+    (syntax-case stx ()
+      ((_ yes)
+       #'(macro-if-u32vector yes (##begin)))
+      ((_ yes no)
+       #'(cond-expand
+          ((or enable-u32vector (not disable-u32vector))
+           yes)
+          (else
+           no))))))
+
+(macro-define-syntax macro-if-s32vector
+  (lambda (stx)
+    (syntax-case stx ()
+      ((_ yes)
+       #'(macro-if-s32vector yes (##begin)))
+      ((_ yes no)
+       #'(cond-expand
+          ((or enable-s32vector (not disable-s32vector))
+           yes)
+          (else
+           no))))))
+
+(macro-define-syntax macro-if-u64vector
+  (lambda (stx)
+    (syntax-case stx ()
+      ((_ yes)
+       #'(macro-if-u64vector yes (##begin)))
+      ((_ yes no)
+       #'(cond-expand
+          ((or enable-u64vector (not disable-u64vector))
+           yes)
+          (else
+           no))))))
+
+(macro-define-syntax macro-if-s64vector
+  (lambda (stx)
+    (syntax-case stx ()
+      ((_ yes)
+       #'(macro-if-s64vector yes (##begin)))
+      ((_ yes no)
+       #'(cond-expand
+          ((or enable-s64vector (not disable-s64vector))
+           yes)
+          (else
+           no))))))
+
+(macro-define-syntax macro-if-f32vector
+  (lambda (stx)
+    (syntax-case stx ()
+      ((_ yes)
+       #'(macro-if-f32vector yes (##begin)))
+      ((_ yes no)
+       #'(cond-expand
+          ((or enable-f32vector (not disable-f32vector))
+           yes)
+          (else
+           no))))))
+
+;;;----------------------------------------------------------------------------
+
 (##include "_kernel#.scm")
+(##include "_fifo#.scm")
 (##include "_thread#.scm")
 (##include "_system#.scm")
 (##include "_num#.scm")
 (##include "_io#.scm")
 (##include "_eval#.scm")
+(##include "_module#.scm")
 (##include "_repl#.scm")
 (##include "_std#.scm")
 (##include "_nonstd#.scm")
