@@ -137,6 +137,22 @@
     ("^macro-check"            . defun)
     ("^macro-force-vars$"      . defun)
     ("^macro-number-dispatch$" . defun)
+    ("^macro-exact-int-dispatch$" . defun)
+    ("^macro-gen"              . defun)
+    ("^macro-make-gen"         . defun)
+    ("^macro-make-code"        . defun)
+    ("^macro-reference-step!"  . defun)
+    ("^macro-call-step!"       . defun)
+    ("^macro-future-step!"     . defun)
+    ("^macro-delay-step!"      . defun)
+    ("^macro-lambda-step!"     . defun)
+    ("^macro-define-step!"     . defun)
+    ("^macro-set!-step!"       . defun)
+    ("^macro-reference-step!"  . defun)
+    ("^macro-constant-step!"   . defun)
+    ("^mk-degen"               . defun)
+    ("macro-force-elem"        . defun)
+    ("macro-check-elem"        . defun)
    ))
 
 ;;;----------------------------------------------------------------------------
@@ -567,6 +583,7 @@ enlarge the window if it is too small."
   ;; Emacs 19 uses a 1-bit flag.  We will have to set up our
   ;; syntax tables differently to handle this.
   ;; Stolen from CC Mode.
+'
   (let ((table (copy-syntax-table))
         entry)
     (modify-syntax-entry ?a ". 12345678" table)
